struct vs_input_t
{
	// we are not defining our own input data; 
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv : TEXCOORD;

	float3 tangent: TANGENT;
	float3 bitangent: BITANGENT;
	float3 normal : NORMAL;
};


//--------------------------------------------------------------------------------------
// Constant Input
// ------
// Constant Input are buffers bound that all shaders share/read.
// It can't be changed from the shader, but can be
// changed in-between draw calls on the CPU
//--------------------------------------------------------------------------------------

// buffer holding time information from our game
cbuffer frame_constants : register(b0)          // constant buffer slot 0
{
	float SYSTEM_TIME;
	float SYSTEM_FRAME_TIME;
	float GAME_TIME;
	float GAME_DELTA_TIME;

	float GAMMA;
	float INVERSE_GAMMA;
	float2 padding2;
};

// buffer telling us information about our camera
cbuffer camera_consts : register(b1)
{
	float4x4 PROJECTION; // CAMERA_TO_CLIP_TRANSFORM
	float4x4 VIEW;		 // WORLD_TO_CAMERA_TRANSFORM

	float3 CAMERA_POSITION;
	float pad00;
};


// information that might change per model/object
cbuffer model_constants : register(b2)         // constant buffer slot 1
{
	float4x4 MODEL;
	float4 TINT;

	float SPECULAR_FACTOR;
	float SPECULAR_POWER;
	float2 pad01;
};

struct light_t
{
	float3 world_position;
	float pad02;  // this is not required, but know the GPU will add this padding to make the next variable 16-byte aligned

	float3 color;
	float intensity; // rgb and an intensity

    float3 direction;  // the direction of light, default (0,0,1)
    float direction_factor; // point light (0.f) or directional light (1.0f)

	float3	attenuation;
	float	dot_inner_angle;

	float3	spec_attenuation;
	float	dot_outer_angle;
};

static const int MAX_LIGHTS = 8;

// buffer telling us information about our camera
cbuffer scene_constants : register(b3)         // constant buffer slot 1
{
	float4 AMBIENT;
	float4 EMISSIVE;

	light_t LIGHTS[MAX_LIGHTS];

	float DIFFUSE_FACTOR;
	float EMISSIVE_FACTOR;
    float NEAR_FOG_DISTANCE;
    float FAR_FOG_DISTANCE;

    float4 NEAR_FOG_COLOR;
    float4 FAR_FOG_COLOR;
};


// Textures & Samplers are also a form of constant
// data - uniform/constant across the entire call
Texture2D <float4> tDiffuse   : register(t0);   // color of the surface
Texture2D <float4> tNormal    : register(t1);   // normal of the surface
SamplerState sSampler : register(s0);           // sampler are rules on how to sample (read) from the texture.

//--------------------------------------------------------------------------------------
// Programmable Shader Stages
//--------------------------------------------------------------------------------------

//--------------------------------------------------------------------------------------
// for passing data from vertex to fragment (v-2-f)
struct v2f_t
{
	float4 position : SV_POSITION;
	float4 color : COLOR;
	float2 uv : UV;

	float3 world_position : WORLD_POSITION;
	float3 world_normal : WORLD_NORMAL;
	float3 world_tangent : WORLD_TANGENT;
	float3 world_bitangent : WORLD_BITANGENT;
};


float3 colorToNormalizedVector(float4 color)
{
	return float3(color.x * 2 - 1, color.y * 2 - 1, color.z * 2 - 1);
}

float4 normalizedVectorToColor(float3 v)
{
    return float4(v.x * 0.5f + 0.5f, v.y * 0.5f + 0.5f, v.z * 0.5f + 0.5f, 1.0f);
}

float2 ComputeLightFactor(light_t light, float3 world_pos, float3 world_normal, float3 dir_to_eye)
{
    float3 vec_to_light = light.world_position - world_pos;

    float dist = length(vec_to_light);
    float3 dir_to_light = normalize(vec_to_light);

    float3 light_dir = lerp(-dir_to_light, light.direction, light.direction_factor);
    float3 att_vec = float3(1.0f, dist, dist * dist);

    float dot_angle = dot(-dir_to_light, light.direction);
    float cone_att = smoothstep( light.dot_outer_angle, light.dot_inner_angle, dot_angle);

    float diff_att = light.intensity / dot(att_vec, light.attenuation) * cone_att;
    float spec_att = light.intensity / dot(att_vec, light.spec_attenuation) * cone_att;

    // Compute Diffuse
    float dot_incident = dot(-light_dir, world_normal);
    float facing = smoothstep(-0.4f, 0.1f, dot_incident);
    float diffuse = max(0.0f, dot_incident);

    // Compute Specular, using blinn-phong
    float3 hv = normalize(-light_dir + dir_to_eye);
    float specular = max(0.0f, dot(world_normal, hv));
    specular = facing * pow(specular, SPECULAR_POWER);

    return float2(diff_att * diffuse, spec_att * specular);
}

float3 ComputeLightingAt(float3 world_pos, float3 world_normal, float3 surf_color)
{
    float3 dir_to_eye = normalize(CAMERA_POSITION - world_pos);

    float3 diffuse = pow(AMBIENT.xyz * AMBIENT.w, GAMMA.xxx);
    float3 spec = (0.f).xxx;

    for (uint i = 0; i < MAX_LIGHTS; i++)
    {
        float3 light_color = LIGHTS[i].color.rgb;
        light_color = pow(light_color, GAMMA.xxx);

        float2 light_factors = ComputeLightFactor(LIGHTS[i], world_pos, world_normal, dir_to_eye);

        diffuse += light_factors.x * light_color;
        spec += light_factors.y * light_color;
    }

    diffuse = min(DIFFUSE_FACTOR * diffuse, 1.0f);
    spec *= SPECULAR_FACTOR;

    return diffuse * surf_color + spec;
}

float4 ApplyFog(float4 inColor, float distance)
{
    float t = (distance - NEAR_FOG_DISTANCE) / (FAR_FOG_DISTANCE - NEAR_FOG_DISTANCE);
    //return float4(t, 0, 0, 1);
    float4 fogTint = lerp(NEAR_FOG_COLOR, FAR_FOG_COLOR, t);
	fogTint = saturate(fogTint);
    return fogTint * inColor;
}
