#include "common.hlsl"

cbuffer dissolve_material_consts : register(b4)
{
	float3 BURN_START_COLOR;
	float BURN_EDGE_WIDTH;
	float3 BURN_END_COLOR;
	float BURN_PROGRESS;
} 

Texture2D <float4> tDissolve  : register(t8);   // dissolve texture

//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}

//--------------------------------------------------------------------------------------
// Fragment Shader
// 
// SV_Target0 at the end means the float4 being returned
// is being drawn to the first bound color target.
float4 FragmentFunction(v2f_t input) : SV_Target0
{
	float burn_value = tDissolve.Sample(sSampler, input.uv).r;
	float burn_min = lerp(-BURN_EDGE_WIDTH, 1.0f, BURN_PROGRESS); 
	float burn_max = burn_min + BURN_EDGE_WIDTH; 
	clip(burn_value - burn_min);

	float burn_mix = smoothstep( burn_min, burn_max, burn_value ); 
	float3 burn_color = lerp( BURN_START_COLOR, BURN_END_COLOR, burn_mix );



	// float height = tDissolve.Sample(sSampler, input.uv).r;
	// clip(0.6f - height);


	// use the uv to sample the texture
	float4 texture_color = tDiffuse.Sample(sSampler, input.uv);
	float3 surface_color = (input.color * texture_color).xyz; // multiply our tint with our texture color to get our final color; 
	float surface_alpha = (input.color.a * texture_color.a);

	surface_color = pow(surface_color, GAMMA.xxx);

	float3 diffuse = AMBIENT.xyz * AMBIENT.w; // ambient color * ambient intensity

	// get my surface normal - this comes from the vertex format
	// We now have a NEW vertex format
	// update normal with the normal texture

	float3 surface_normal = colorToNormalizedVector(tNormal.Sample(sSampler, input.uv));

    float3 normal = normalize(input.world_normal); 
    float3 tangent = normalize(input.world_tangent);
    float3 bitangent = normalize(input.world_bitangent); 

	float3x3 tbn = float3x3( tangent, bitangent, normal );
	float3 world_normal = mul( surface_normal, tbn );
	
	float3 final_color = ComputeLightingAt( input.world_position, world_normal, surface_color);

    final_color = lerp( burn_color, final_color, burn_mix ); 
	final_color = pow( final_color, INVERSE_GAMMA.xxx ); 

	float4 fColor = ApplyFog(float4(final_color.rgb, 1), length(input.world_position - CAMERA_POSITION));

	return float4(final_color.rgb, surface_alpha * fColor.a);
}