#include "common.hlsl"

struct fragment_output_t
{
	float4 color : SV_Target0;
	float4 bloom : SV_Target1;
	float4 normal : SV_Target2;
	float4 albedo : SV_Target3;
	float4 tangent : SV_Target4;
};

//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	world_normal = float4(cross(world_tangent.xyz, world_bitangent.xyz), 0);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}

//--------------------------------------------------------------------------------------
// Fragment Shader
// 
// SV_Target0 at the end means the float4 being returned
// is being drawn to the first bound color target.
fragment_output_t FragmentFunction(v2f_t input)
{
	// use the uv to sample the texture
	float4 texture_color = tDiffuse.Sample(sSampler, input.uv);
	float3 surface_color = (input.color * texture_color).xyz; // multiply our tint with our texture color to get our final color; 
	float surface_alpha = (input.color.a * texture_color.a);

	surface_color = pow(surface_color, GAMMA.xxx);

	float3 diffuse = AMBIENT.xyz * AMBIENT.w; // ambient color * ambient intensity

	// get my surface normal - this comes from the vertex format
	// We now have a NEW vertex format
	// update normal with the normal texture

	float3 surface_normal = colorToNormalizedVector(tNormal.Sample(sSampler, input.uv));

    float3 normal = normalize(input.world_normal); 
    float3 tangent = normalize(input.world_tangent);
    float3 bitangent = normalize(input.world_bitangent); 

	float3x3 tbn = float3x3( tangent, bitangent, normal );
	float3 world_normal = mul( surface_normal, tbn );
	
	float3 final_color = ComputeLightingAt( input.world_position, world_normal, surface_color);

	float3 overflow = max(final_color.rgb - float3(1, 1, 1), float3(0,0,0));
	
	final_color = pow( final_color, INVERSE_GAMMA.xxx ); 
	

	float4 fColor = ApplyFog(float4(final_color.rgb, 1), length(input.world_position - CAMERA_POSITION));

	fragment_output_t output;
	output.color = float4(final_color.rgb, surface_alpha * fColor.a);
	output.bloom = float4(overflow, 1);
	output.normal = normalizedVectorToColor(world_normal);
	output.albedo = texture_color;
	output.tangent = normalizedVectorToColor(tangent);

	return output;
}