#include "common.hlsl"

cbuffer parallax_consts : register(b4)
{
	uint STEP_COUNT;
	float DEPTH;
	float2 parallax_pad00;
} 

Texture2D <float4> tHeightmap  : register(t8);   // Height map texture


//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}

float2 ComputeShallowParallaxUV(float3 local_dir, float2 uv_start)
{
	float2 uv_end = uv_start + local_dir.xy * float2(DEPTH, -DEPTH);

	float h0 = tHeightmap.Sample(sSampler, uv_start).x;
	float h1 = tHeightmap.Sample(sSampler, uv_end).x;

	float t = (1 - h0) / ((h1 - h0) + 1);
	return lerp(uv_start, uv_end, t);
}

float2 ComputeDeepParallaxUV(float3 local_dir, float2 uv_start)
{
	float2 uv = uv_start;

	const float step = 1.0f / (STEP_COUNT * 1.0f);
	float2 uv_shift_step =  local_dir.xy * step * DEPTH * 0.5f;

	float z = 1.0f;
	float height = 0.0f;

	uint i = 0;

	float heights[32];

	[unroll(32)]
	for (i = 0; i < STEP_COUNT; i++)
	{
		heights[i] = tHeightmap.Sample(sSampler, uv).x;
		uv += uv_shift_step;
	}

	[unroll(32)]
	for (i = 1; i < STEP_COUNT; i++)
	{
		float h1 = heights[i];

		if ((z - step) <= h1)
		{
			float2 uv1 = uv_start + i * uv_shift_step;
			float2 uv0 = uv1 - uv_shift_step;

			float h0 = heights[i - 1];
			float dh = h1 - h0;

			float t = ((h0 - z) / (-step - dh));
			return lerp(uv0, uv1, t);
		}
		z -= step;
	}
	
	return uv;
}

float4 FragmentFunction(v2f_t input) : SV_Target0
{
    float3 normal = normalize(input.world_normal); 
    float3 tangent = normalize(input.world_tangent);
    float3 bitangent = normalize(input.world_bitangent); 

	float3x3 tbn = float3x3( tangent, bitangent, normal );
	
	// calculating uv for parallax
	float3 look_dir = normalize(input.world_position - CAMERA_POSITION);
	float3 surf_look_dir = mul( tbn, look_dir );
	
	float2 uv = ComputeDeepParallaxUV(surf_look_dir, input.uv);
	// return normalizedVectorToColor(normal);
	
	// use the uv to sample the texture
	float4 texture_color = tDiffuse.Sample(sSampler, uv);
	float3 surface_color = (input.color * texture_color).xyz; // multiply our tint with our texture color to get our final color; 
	float surface_alpha = (input.color.a * texture_color.a);

	surface_color = pow(surface_color, GAMMA.xxx);

	float3 diffuse = AMBIENT.xyz * AMBIENT.w; // ambient color * ambient intensity

	// get my surface normal - this comes from the vertex format
	// We now have a NEW vertex format
	// update normal with the normal texture

	float3 surface_normal = colorToNormalizedVector(tNormal.Sample(sSampler, uv));
	float3 world_normal = mul( surface_normal, tbn );

	
	float3 final_color = ComputeLightingAt( input.world_position, world_normal, surface_color);

	final_color = pow( final_color, INVERSE_GAMMA.xxx ); 

	float4 fColor = ApplyFog(float4(final_color.rgb, 1), length(input.world_position - CAMERA_POSITION));

	return float4(final_color.rgb, surface_alpha * fColor.a);
}